Storm Benchmarking Scripts
==============

This package contains logdata and replication scripts for the evaluation part of the paper:
	"The Probabilistic Model Checker Storm" by Christian Hensel, Sebastian Junges, Joost-Pieter Katoen, Tim Quatmann, and Matthias Volk

# Requirements

The scripts require Python 3 and a working installation of Storm

# Data of Experiments
- The directory `qcomp` contains all model and property data of our experiments. In fact, this is a snapshot of the [QComp git repository](https://github.com/ahartmanns/qcomp.git) (commit 694bb13755cef501e783efbf51215fe3ab9d8dbe). We added the three MDP model files (converted_firewire-pta.jani, converted_wlan-large.jani, and converted_zeroconf-pta.jani) that were generated from PTA models using moconv (as described in the paper).
- The directory `results/logs` contains all execution results including logfiles. Open the file `results/table/table.html` in a browser to browse this data comfortably.
- The directory `results/plots` contains the csv data and LaTeX code that generate the plots from the paper.

# Replication
- Run `python3 run.py` and follow the displayed instructions to create a list of invocations you would like to execute.
- Afterwards, run `python3 postprocess.py` to create an HTML table and .csv files similar to the ones we provide in the `results` directory.

# Further information

##Terminology

- Benchmark: A benchmark is a combination of a model, a parameter assignment, and a single property. Each benchmark has a unique identifier (e.g. `consensus.4-2.disagree`) consisting of the short model name, the parameter values (in the order they appear on the website), and the name of the property.
- Configuration: A way to invoke the tool, e.g., by selecting a specific engine or enabling a specific feature
- Invocation: An invocation is a sequence of command lines that produce the result of a single given benchmark by invocing the tool. We consider *sequences* of command lines to allow for, e.g., conversions to other formalisms before executing the actual tool.
- Execution: An execution is the result of executing an invocation.

## Overview of Files

- `run.py` selects which experiments are being executed and performs the actual execution.
- `postprocess.py` processes the data from tool executions and exports them in various formats
- `internal/benchmark.py` provides access to the data associated with a benchmark (including the meta data as stored on qcomp.org).
- `internal/configuration.py` provides access to the data associated with a configuration.
- `internal/execution.py` provides access to the data associated with an execution. 
- `internal/export.py` various auxiliary functions that produce the desired .csv or .html files during postprocessing. 
- `internal/invocation.py` provides access to the data associated with an invocation.
- `internal/qcomp_2019_benchmarks.csv` contains all 100 benchmarks for QCOMP 2019
- `internal/qcomp_2019_benchmarks_compatible.csv` contains the 96 benchmarks for QCOMP 2019 that are compatible with Storm
- `internal/storm.py` contains methods specific for Storm, e.g., to parse log files or to create command line arguments
- `internal/utility.py` provides various utility functions.


