from internal.export import *
from internal.utility import *
import sys
import os


if __name__ == "__main__":
    print("Storm benchmarking tool.")
    print("This script gathers data of executions and exports them in various ways.")
    print("Usages:")
    print("python3 {}    reads the log file directory from the default settings file".format(sys.argv[0]))
    print("python3 {} path/to/first/logfiles/ path/to/second/logfiles/ ...    reads from multiple log file directories '".format(sys.argv[0]))
    print("")
    if (len(sys.argv) == 2 and sys.argv[1] in ["-h", "-help", "--help"]):
        exit(1)

    settings = Settings()

    if len(sys.argv) == 1:
        logdirs = [settings.logs_dir()]
    else:
        logdirs = sys.argv[1:]


    # Gather execution data
    exec_data = OrderedDict() # Tool -> Config -> Benchmark -> Data
    benchmark_ids = []
    tools_configs = OrderedDict() # Tool X Config -> Num Supported
    for logdir_input in logdirs:
        logdir = os.path.expanduser(logdir_input)
        if not os.path.isdir(logdir):
            print("Error: Directory '{}' does not exist.".format(logdir))
    
        for res_json in [ load_json(os.path.join(logdir, f)) for f in os.listdir(logdir) if f.endswith(".json") and os.path.isfile(os.path.join(logdir, f)) ]:
            tool = res_json["tool"]
            config = res_json["configuration-id"]
            benchmark = res_json["benchmark-id"]
            exec_data.setdefault(tool, OrderedDict())
            exec_data[tool].setdefault(config, OrderedDict())
            if benchmark in exec_data[tool][config]:
                print("Error: Multiple result files found for {}.{}.{}".format(tool,config,benchmark))
            res_json["log"] = os.path.join(logdir, res_json["log"])
            exec_data[tool][config][benchmark] = res_json
            if not benchmark in benchmark_ids:
                benchmark_ids.append(benchmark)
            tools_configs.setdefault((tool,config), 0)
            if not is_not_supported(res_json):
                tools_configs[(tool,config)] += 1
    
    benchmark_ids.sort()
    print("Found Data for {} benchmarks and {} tool-config combinations".format(len(benchmark_ids), len(tools_configs)))
    print("Number of supported benchmarks (of {}):".format(len(benchmark_ids)))
    for (t,c) in tools_configs:
        print("{}.{}: {}".format(t,c,tools_configs[(t,c)]))
        
    tools_configs_sorted = []
    for engine in ["sparse", "hybrid", "dd", "ddbisim", "sound", "exact"]:
        for (t,c) in tools_configs:
            if engine == c:
                tools_configs_sorted.append((t,c))
    for (t,c) in tools_configs:
        if (t,c) not in tools_configs_sorted:
            tools_configs_sorted.append((t,c))
        
        
    print("Generating file {} for scatter plots".format(settings.results_file_scatter()))
    scatter_csv = generate_scatter_csv(settings, exec_data, benchmark_ids, tools_configs_sorted)
    save_csv(scatter_csv, settings.results_file_scatter())
    print("Generating file {} for quantile plots".format(settings.results_file_quantile()))
    quantile_csv = generate_quantile_csv(settings, exec_data, benchmark_ids, tools_configs_sorted)
    save_csv(quantile_csv, settings.results_file_quantile())
    print("Generating interactive html table in directory {}".format(settings.results_dir_table()))
    generate_table(settings, exec_data, benchmark_ids, tools_configs_sorted, settings.results_dir_table())
    print("-" * 30 + "\nSummary of results:\n")
    print(generate_summary_table(settings, exec_data, benchmark_ids, tools_configs_sorted))


